import re
import hashlib
import string
import subprocess
import requests
import logging
from . import ono
from . import yota

function_logger = logging.getLogger("http_logger")
fn_fh = logging.FileHandler('mb_func.log', mode='a')
fn_fh.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s:%(name)s:%(levelname)s:%(message)s')
fn_fh.setFormatter(formatter)
function_logger.addHandler(fn_fh)
function_logger.setLevel(logging.DEBUG)


def parse_http_string(inputstr):
    
    def get_title(instr):
        title_match = re.search(r"\.([A-Z][^\.]+)", instr)
        if title_match:
            title = title_match.group(1)
            rest_str = inputstr.replace(title + ".", "")
            if rest_str == inputstr:
                rest_str = inputstr.replace(title, "")
            return(title, rest_str)
        else:
            return None, instr
        
    def get_url(instr):
        url_match = re.search(r'h\.(.+/[^\.]+)', instr)
        rest = instr.replace('h.' + url_match.group(1), "")
        url = 'http://' + url_match.group(1)
        return url, rest
    
    def get_tags(instr):
        result = []
        elements = instr.split(".")
        for item in elements:
            if len(item) > 0:
                result.append(item)
        return result
    
    title, rest = get_title(inputstr)
    url, rest2 = get_url(rest)
    tags = get_tags(rest2)
    
    return url, title, tags
    
def remove_http(instr):
    m = re.search(r"(https?://)", instr)
    if m:
        new_str = instr.replace(m.group(1),"")
    return new_str

class Http:

    def __init__(self, url, tags=[], title=None):
        self.url = url
        if tags:
            self.tags = tags
        else:
            self.tags = []
        if title:
            self.title = title
        else:
            self.title = ""

        file_endings = ['com','org','net','dk','txt','php','htm','html','py','omm','zip','mp3','mp4','mkv','avi','gif','jpg','jpeg','rar','asc','pgp']
        def parse_links_w_file_ending(self):
            
            for item in file_endings:
                if self.tags:
                    if item in self.tags[0]:
                        self.url += "." + self.tags[0]
                        self.tags = self.tags[1:]
                        #break

        parse_links_w_file_ending(self)

        stripped_url = remove_http(self.url)
        self.omm = "h." + stripped_url
        
        if self.title:
            self.omm += "." + self.title.replace(" ","_")
        
        if self.tags:
            for item in self.tags:
                self.omm += "." + item
        
        self.first_name = self.hash()[:3]

        if self.tags and self.title:
            self.html = '<a href="' + str(self.url) + '" title="' + " ".join(tags) +  '">' + str(self.title.replace("_"," ")) + '</a>'
  
        elif self.title:
            self.html = '<a href="' + str(self.url) + '">' + str(self.title) + '</a>'

        else:
            self.html = '<a href="' + str(self.url) + '" title="' + str(self.tags) + '"> MyYota</a>'

    def __repr__(self):
        """Defines internal print() format (internal method)."""

        tag_string = "  ("                      # build tag string
        if self.tags:
            for i, tag in enumerate(self.tags):
                tag_string += str(tag)
                if (i + 1) < len(self.tags):
                    tag_string += ', '

        tag_string += ')'

        #result = ""
        if self.title:
            title = self.title
        else:
            title = "MyHttp"
        
        result = u"\U0001F517" + " " + title

        if self.tags:
            tag_string = "  ("                      # build tag string
            for i, tag in enumerate(self.tags):
                tag_string += str(tag)
                if (i + 1) < len(self.tags):
                    tag_string += ', '
            tag_string += ')'
        else:
            tag_string = ""
        
        result += tag_string

        # if self.bits:
        #     result += '  ['
        #     for item in self.bits:
        #         result += item.title + ', '

        #     result = result[:-2] + ']'

        my_length = 60

        if (my_length - len(title) - len(tag_string) + 2) < 0:
            title_length = 60 - len(title) - len(tag_string) - 3
            new_title = title[:title_length].replace("_"," ") + "..."
            result = result.replace(title, new_title)


        result += " " * (my_length - len(title) - len(tag_string)) + " o." + self.hash()
        
        return(result)

    def __str__(self):
        
        return self.omm

    def __add__(self, value):

        if isinstance(value, Http):
            newMix = yota.Mixtape(self)
            newMix += value
            return newMix


    def hash(self):
        m = hashlib.sha256()
        m.update(self.omm.encode())
        temp_hash = m.hexdigest()
        for i, char in enumerate(temp_hash):
            if char in string.ascii_letters:
                calculated_hash = temp_hash[i:i+11]
                return(calculated_hash)

    def open(self):
        subprocess.Popen(['xdg-open',self.url]).wait()
    
    
    def html_raw(self):
        r = requests.get(self.url)
        return r.text

    def html_grep(self, regex_str):
        class ListWithGrep(list):
            
            def grep(self, keyword, exclude=None):

                def excluded(inputstr):
                    for item in exclude:
                        if item in inputstr:
                            return True
                        else:
                            r

                result = []
                for item in self:
                    m = re.search(keyword, item)
                    if m:
                        if exclude:
                            exclu = excluded(item)
                            if not exclu:
                                return result

        
        content = self.html_raw()
        m = re.findall(regex_str, content)
        result = ListWithGrep()
        if m:
            for item in m:
                result.append(item)
            return result

    def links(self):
        result_list = self.html_grep("\"(http.+?)\"")
        return result_list

    def vlc(self):
        subprocess.Popen(['vlc', self.url])



def main(instr):
    url, title, tags = parse_http_string(instr)
    newHttpObj = Http(url=url, title=title, tags=tags)
    ono.add_to_hash_dict(newHttpObj.omm)
    function_logger.debug(f"called main() with arg {instr}, result {newHttpObj.url}")
    return newHttpObj




import sys

def get_site_packages_path():
    """"Returns the first sys.path 'site-packages' or 'dist-packages' match."""
    #print(f'sys.path: {sys.path}')
    for path in sys.path: 
        if 'site-packages' in path or 'dist-packages' in path:
            if flask_server == False:  # client execution environment
                if '.local' not in path: # so exclude .local result from sys.path
                    new_path = path + os_sep + package_name
                    return new_path
            else: # server execution, so don't exclude .local result from sys.path
                new_path = path + os_sep + package_name
                return(new_path) 

def get_os_file_separator():
    """Returns OS-dependent file separator character (Windows or other)."""
    if platform == 'win32':
        os_sep = '\\'
    else:
        os_sep = '/'
    return(os_sep)


# hard-coded canonical package version number

version_number = 'v0.9.1.5'

summary = """Online Media Metadata (OMM) is an original concept to succinctly reference online media ressources in mediabytes.

That is, by URL (h.url.com/subpage), platform-specific hash (y.youtubehash), bitly link hash (b.bitlyhash) or 
mediabyte hash (o.mediabytehash) with optional explicit meta-data (title and tags) and YouTube format time code support 
for YouTube videos, MP3s and m3u8 streams.

Everything parsed by the local system can be referenced locally by o.mediabytehash using enough characters to uniquely match, e.g. o.e88r.

Everything parsed online at skillporn.tv and skillporn.tv/search is also available in client instances using the full o.mediabytehash."""


# constants for use in modules

flask_server = False
if version_number[-3:] == 'pro':
    package_name = 'mb'
else:
    package_name = 'mediabyte'
platform = sys.platform
os_sep = get_os_file_separator()
package_path = get_site_packages_path()
srt_folder_path = package_path + os_sep + 'srt'
hash_dict_path = package_path + os_sep + 'files' + os_sep + 'hash_dict.json'
filters_path = package_path + os_sep + 'files' + os_sep + 'filters.json'

header_str = 'Online Media Metadata (OMM) ' + version_number.split("-")[0]





#!/home/dd/anaconda3/bin/python
import re
import requests
import logging
from . import cnf

function_logger = logging.getLogger("online_requests_logger")
fn_fh = logging.FileHandler('mb_func.log', mode='a')
fn_fh.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s:%(name)s:%(levelname)s:%(message)s')
fn_fh.setFormatter(formatter)
function_logger.addHandler(fn_fh)
function_logger.setLevel(logging.DEBUG)


def main(url, omm_function):

    def parse_yno_file_readlines(content):
        """parse .omm readlines list for yotas spanning 
        several lines, return list of yota strings."""

        parsed_lines = []
        for line in content:
                line = line.strip()
                if len(line) > 2:  # filter empty lines
                    m0 = re.search('^#', line)
                    if not m0: # filter comments
                        m = re.search('^\.', line)
                        if m: # concatinate multiple line yotas
                            parsed_lines[-1] += line
                        else:
                            parsed_lines.append(line)
        return(parsed_lines)

    # use 'OMM v0.9.0.4'-style user-agent
    headers = {'User-Agent': cnf.header_str}
    r = requests.get(url, headers=headers)
    function_logger.debug(f"downloaded .omm Mixtape link {url}")
    file_content = r.text.split("\n")

    filelines2 = parse_yno_file_readlines(file_content)
    omm = omm_function

    if len(filelines2) > 1:
        mixtape = omm(filelines2[0]) + omm(filelines2[1])
    else:
        mixtape = omm(filelines2[0])

    for line in filelines2[2:]:
        mixtape += omm(line)

    return(mixtape)




#!/home/dd/anaconda3/bin/python
import os, re, json, hashlib, string, requests, logging
from . import lib
from . import cnf

logger = logging.getLogger("ono_logger")
fn_fh = logging.FileHandler('mb_func.log', mode='a')
fn_fh.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s:%(name)s:%(levelname)s:%(message)s')
fn_fh.setFormatter(formatter)
logger.addHandler(fn_fh)
logger.setLevel(logging.DEBUG)


hash_dict_filename = cnf.package_path + cnf.os_sep + 'files' + cnf.os_sep + 'hash_dict.json'

if not os.path.isfile(hash_dict_filename):
    tempDict = {}
    with open(hash_dict_filename, 'w') as f:
        json.dump(tempDict, f)
        logger.debug("created new hash_dict")


def write_hash_dict(hash_dict):

    with open('hash_dict.json','w') as f:
        json.dump(hash_dict, f)
        logger.debug("wrote hash_dict")


def load_hash_dict():
    with open(hash_dict_filename, 'r') as f:
        hash_dict = json.load(f)
    return(hash_dict)


def add_to_hash_dict(mediabyte_str):
    """Add Yota/Cue/Sample/bit string to hash_dict JSON."""

    with open(hash_dict_filename, 'r') as f:
        hash_dict = json.load(f)
        original_hash_dict_len = len(hash_dict)

    def hash(mediabyte_str):
        m = hashlib.sha256()
        m.update(mediabyte_str.encode())
        temp_hash = m.hexdigest()
        for i, char in enumerate(temp_hash):
            if char in string.ascii_letters:
                calculated_hash = temp_hash[i:i+11]
                return(calculated_hash)

    no_whitespace_mediabyte_str = mediabyte_str.replace(' ', '_')
    calculated_hash = hash(no_whitespace_mediabyte_str)
    
    #if not mixtape_check:
    hash_dict[calculated_hash] = mediabyte_str.replace(' ','_')
    
    if len(hash_dict) != original_hash_dict_len:
        with open(hash_dict_filename, 'w') as f:
            json.dump(hash_dict, f)
        logger.debug(f"wrote {mediabyte_str} to hash_dict")    
        

def check_ono(ono_str, string_arg=False):
    with open(hash_dict_filename, 'r') as f:
        hash_dict = json.load(f)
    
    keys = list(hash_dict.keys())
    m = re.search('^o\.([a-zA-Z0-9]+)', ono_str)
    o_hash = m.group(1)
    results = []
    for key in keys:
        if o_hash == key[:len(o_hash)]:
            if not string_arg:
                myObject = lib.Convert.omm(hash_dict[key])
                results.append(myObject)
            else:
                results.append(hash_dict[key])
    if len(results) > 1:
        return "Multiple matches, please add characters and try again"
    elif len(results) == 1:
        logger.debug(f"checked hash_dict for {ono_str}, found {results[0]}")
        return results[0]
    elif len(results) == 0:
        if cnf.flask_server == False:
            url_get = "http://skillporn.tv/check/" + ono_str[2:]
            # use 'OMM v0.9.0.4'-style user-agent
            headers = {'User-Agent': cnf.header_str}
            r = requests.get(url_get, headers=headers)
            logger.debug(f"checking server skillporn.tv/check/ for {ono_str}")
            if r:
                logger.debug(f"success, server answers {r.text}")
                return lib.Convert.omm(r.text)



hash_dict = load_hash_dict()


import hashlib, string

class Message:
    def __init__(self, to, title, tags=[]):
        self.name = to
        self.title = title
        if tags:
            self.tags = tags
        else:
            self.tags = []
            
        self.omm = "m." + self.name + "." + self.title
        if tags:
            for tag in tags:
                self.omm += "." + tag

        self.first_name = self.hash()[:3]
    
    def hash(self):
        m = hashlib.sha256()
        m.update(self.omm.encode())
        temp_hash = m.hexdigest()
        for i, char in enumerate(temp_hash):
            if char in string.ascii_letters:
                calculated_hash = temp_hash[i:i+11]
                return(calculated_hash)

    def __repr__(self):
        if not self.tags:
            result = "\"" + self.title + "\"  " + "@" + self.name + " " * (53 - (len(self.title) + len(self.name)))
            result += "   " + "o." + self.hash()
        else:
            tags_length = 0
            for tag in self.tags:
                tags_length += len(tag)
            result = "\"" + self.title + "\"  " + "@" + self.name + " " * (49 - (len(self.title) + len(self.name) + tags_length))
            if self.tags:
                tag_str = "  ("
                if self.tags:
                    for tag in self.tags:
                        tag_str += tag + ", "
                    tag_str = tag_str[:-2] + ")"
                result += tag_str
            result += "   " + "o." + self.hash()

        return result

    def __str__(self):
        return self.omm
        
def parse(inputstr):
    tags = inputstr.split(".")
    if tags[0] != 'm':
        raise ValueError("string does not conform to standard (does not start with 'm.')")
    else:
        receiver = tags[1]
        text_body = tags[2]
        if len(tags) > 3:
            my_tags = tags[3:]
        else:
            my_tags = []

    myMessage = Message(receiver, text_body, my_tags)
    return myMessage